package LinkClean;
use Exporter;

@ISA = ('Exporter');
@EXPORT = qw(user normalize domain qdomain topdomain loadmappings ytvid);

use strict;

my %vox_mapping;
my %myspace_mapping;

sub loadmappings {
  open MAP, "< vox-mapping.txt";
  while (<MAP>) {
    (my $src, my $dst) = split;
    $vox_mapping{$src} = $dst;
  }
  close MAP;

  open MAP, "< myspace-mapping.txt";
  while (<MAP>) {
    (my $src, my $dst) = split;
    $myspace_mapping{$src} = $dst;
  }
  close MAP;
}

my @special = qw/ac co com org net/;
my %special;

for my $k (@special) { $special{$k} = 1; }

my %whitelist = (
  '\.spaces\.live\.com$'  => '^(www|c\.services)\.spaces\.live\.com$',
  '\.wordpress\.com$'     => '(^|\.)(www|faq|search|forums|feeds|stats|news|..?)\.wordpress\.com$',
  '\.exblog\.jp$'         => '^(www|pds|md)\.exblog\.jp$',
  '\.livejournal\.com$'   => '^(www|stat|syndicated|p-userpic|community|communities|users|pics|p-stat)\.livejournal\.com$',
  '\.blogspot\.com$'      => '^(www|.?\.bp)\.blogspot\.com$',
  '\.vox\.com$'           => '^(www|a\d)\.vox\.com$',
  '\.yculblog\.com$'      => '^(www|sta|foto|blog)\.yculblog\.com$',
  '\.blogfa\.com$'        => '^www\.blogfa\.com$',
  '\.typepad\.com$'       => '^www\.typepad\.com$',
  '\.blog\.com$'          => '^(www|amadeo|editor)\.blog\.com$',
  '\.over-blog\.com$'     => '(^|\.)(www|idata|mettre-put-idata).over-blog.com$',
  '\.cocolog-nifty\.com$' => '(^|\.)(www|emojies|idata|mettre-put-idata).cocolog-nifty.com$',
  '\.blogs\.com$'         => '^www\.blogs\.com$',
  '\.canalblog\.com$'     => '^(www|storage)\.canalblog\.com$',
);

my @whitelist = keys %whitelist;

my $myspace_url = "blogs.myspace.com/index.cfm?fuseaction=blog&FriendID=";

sub user {
  my $url = normalize(shift @_);
  my $dom = qdomain($url);

  return undef unless defined $dom;

  if ($dom =~ m/(^|\.)myspace\.com$/) {
    my $usr = (split '[/\\\\]', $url, 2)[1]; # take the path
    return undef unless defined $usr;

    ($usr, my $pth) = split /[^\w\d.]/, $usr, 2; # take the first path element
    return undef unless defined $usr;
    return undef if $usr eq "";

    if ($usr eq "index.cfm") {
      return undef unless (defined $pth) && ($pth =~ m/\bfriendid=(\d+)/);
      return exists $myspace_mapping{$1} ? $myspace_mapping{$1} : "$myspace_url$1";
    }

    $dom = "blogs.myspace.com" if $dom eq "myspace.com";
    if ($dom =~ m/^(www|blogs?|profile|latino|..)\.myspace\.com$/) {
      return undef if $usr =~ /\./;      # user names don't have dots
      return undef if $usr eq "modules"; # 'modules' is not an user
      return "blogs.myspace.com/$usr";
    }

    return undef;
  }

  if ($dom =~ m/(^|\.)livejournal\.com$/) {
    if ($url =~ m%^(pics|users|communities|community).livejournal.com/([^/]+)(/|$)%) {
      $dom = "$2.livejournal.com";
    } elsif ($url =~ m%(^|\.)livejournal\.com/(users|community)/([^/]+)(/|$)%) {
      $dom = "$3.livejournal.com";
    } elsif ($url =~ m%(^|\.)livejournal\.com/~([^/]+)(/|$)%) {
      $dom = "$2.livejournal.com";
    }
    return undef if $dom =~ m/[&;]/;
    $dom =~ s/_/-/g;
  }

  if ($dom =~ m/\.vox\.com$/) {
    $dom = $vox_mapping{$dom} if exists $vox_mapping{$dom};
  }

  $dom =~ s/^www\.//; # sites discard www prefix when refering to users
  $dom =~ s/\.files\.wordpress\.com$/.wordpress.com/;
  $dom =~ s/\.ycool\.com$/.yculblog.com/;

  for my $pat (@whitelist) {
    if ($dom =~ m/$pat/) {
      return $dom =~ m/$whitelist{$pat}/ ? undef : $dom;
    }
  }

  return undef;
}

sub normalize {
  my $url = shift @_;
  return undef unless defined $url;
  $url =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
  $url = lc($url);
  $url =~ s/\s//g;
  return undef unless $url =~ m|^https?://|;
  $url =~ s|^https?://||; # get rid of prefix

  return $url;
}

sub domain {
  return qdomain(normalize(shift @_));
}

sub qdomain {
  my $url = shift @_;
  return undef unless defined $url;

  $url =~ s|[^\w\d._-].*$||; # get rid of anything that doesn't look like part of the domain
  die if $url =~ m|[^\w\d._-]|;
  $url =~ s/\.$//; # remove a trailing dot

  return undef unless $url =~ m/.\../; # it should have a dot somewhere in the middle
  return $url;
}

sub topdomain {
  my $dom = $_[0];
  return undef unless defined $dom;

  my @pts = split "\\.", $dom;
  return $dom unless @pts > 1;

  my $main = "$pts[-2].$pts[-1]";

  $main = "$pts[-3].$main" if exists $special{$pts[-2]} && defined $pts[-3];

  return $main;
}

sub ytvid {
  my $link = $_[0];
  return undef unless defined $link;
  $link =~ s|^https?://||; # get rid of prefix
  $link =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
  $link =~ s|&amp;|&|g;
  (my $dom, my $path) = split "/", $link, 2;
  $dom = lc($dom);
  return undef unless defined $path && ($path !~ m/^\s*$/);
  return undef unless $dom =~ m[(^|\.)youtube\.(com|com\.br|fr|jp|nl|pl|es|ir|co\.uk|de)$];

  if ($path =~ m/[&?]v(ideo_id)?=(.*?)([&?#]|$)/) {
    return $2 if length($2) == 11;
  } elsif ($path =~ m[^vi?/(.*?)([&/]|$)]) {
    return $1 if length($1) == 11;
  }

  return undef;
}

1;