#!/usr/bin/perl -w

use strict;
use Encode;
use HTML::Entities;
use LinkClean;

loadmappings;

my @fields = qw/dom user url gid time lang wcnt self numfrnd numilnk numelnk/;
my %post;

while (<>) {
  $post{gid}   = $1 if m/<post:resource_guid>(.*)<\/post:resource_guid>/;
  $post{url}   = $1 if m/<link>(.*)<\/link>/;
  $post{src}   = $1 if m/<dc:source>(.*)<\/dc:source>/;
  $post{tmpub} = $1 if m/<atom:published>(.*)<\/atom:published>/;
  $post{tmfnd} = $1 if m/<post:date_found>(.*)<\/post:date_found>/;
  $post{lang}  = $1 if m/<dc:lang>(.*)<\/dc:lang>/;

  if (m/<description>/) {
    $post{text} = $_;
    while (<>) {
      $post{text} .= " " . $_;
      last if m/<\/description>/;
    }
  }

  if (m/<\/item>/) {
    &process_post;
    %post = ();
  }
}

sub dump_data {
  while ((my $k, my $v) = each %post) {
    print STDERR "$k: $v\n";
  }
  die;
}

sub process_post {
  return unless defined $post{url};

  $post{url}  = "$post{src}/$post{url}" unless $post{url} =~ m|^https?://|;
  $post{user} = user($post{url});

  return unless defined $post{user};

  $post{dom} = topdomain(domain($post{url}));

  $post{time} = $post{tmpub} =~ m/^\s*$/ ? $post{tmfnd} : $post{tmpub};
  &dump_data if $post{time} =~ m/^\s*$/;

  $post{text} =~ s/<\/?description>//g;
  $post{text} = decode_entities($post{text});

  my @inlinks;
  my @exlinks;
  my %friends;

  $post{self} = 0;
  while ($post{text} =~ m/\b(href|src)=["'](.*?)['"]/g) {
    my $link = $2;
    next unless $link =~ m|^https?://|;

    my $friend = user($link);
    if (defined $friend) {
      if ($friend eq $post{user}) {
        $post{self} = 1;
      } else {
        $friends{$friend} = 1;
      }
      push @inlinks, $link;
    } else {
      push @exlinks, $link;
    }
  }

  my @friends = keys %friends;

  $post{numfrnd} = scalar @friends;
  $post{numilnk} = scalar @inlinks;
  $post{numelnk} = scalar @exlinks;

  $post{text} = decode('UTF-8', $post{text});
  $post{text} = decode_entities($post{text});
  $post{text} =~ s|<.*?>| |g;
  $post{text} =~ s|\W+| |g;
  $post{text} =~ s|^ ||g;
  $post{text} =~ s| $||g;
  my @words = split " ", $post{text};
  $post{wcnt} = scalar @words;

  print join(" ", map { $post{$_} } @fields), " ",
        join(" ", @friends), " + ", join(" ", @inlinks), " + ", join(" ", @exlinks), "\n";
#  print encode('UTF-8', $post{text}), "\n";
}
